from texifast.helpers import refine_math_block


def test_refine_math_block() -> None:
    text = "This is a math block $$x^2$$ and this is another math block $$y^2$$"
    assert (
        refine_math_block(text)
        == "This is a math block\n\n$$\nx^2\n$$\n\nand this is another math block$$\ny^2\n$$"
    )



from __future__ import annotations

from typing import TYPE_CHECKING

import numpy as np
import onnxruntime as ort

from .helpers import logger

if TYPE_CHECKING:
    from collections.abc import Sequence
    from os import PathLike
    from typing import Any


class TxfModel:
    def __init__(
        self,
        encoder_model_path: str | bytes | PathLike[Any],
        decoder_model_path: str | bytes | PathLike[Any],
        provider: Sequence[str | tuple[str, dict[Any, Any]]] | None = None,
        session_options: ort.SessionOptions  # pyright: ignore[reportUnknownParameterType]
        | None = None,
        provider_options: Sequence[dict[Any, Any]] | None = None,
        use_io_binding: bool | None = None,
        **kwargs: Any,
    ) -> None:
        """Initialize the TxfModel class.

        You should pass in same type(quantized/fp16) of encoder and decoder models, do not mix
        them. And for CUDAExecutionProvider, it is recommended to use float32 or float16 models
        instead of quantized models.

        Note:
            On CPU, enable I/O binding will bring a little performance degredation(~5%), because the
            time saved from memory management is offset by Python's for loop, so it is disabled by
            default.
            On CUDA, enable I/O binding is a no-brainer, it will bring huge performance boost(~2x),
            and it is enabled by default.

        Args:
            encoder_model_path (str | bytes | PathLike): Path to the encoder model.
            decoder_model_path (str | bytes | PathLike): Path to the decoder model.
            provider (Sequence[str | tuple[str, dict[Any, Any]]], optional): Providers for the model. Defaults to None.
            session_options (ort.SessionOptions, optional): Session options for the model. Defaults to None.
            provider_options (Sequence[dict[Any, Any]], optional): Provider options for the model. Defaults to None.
            use_io_binding (bool, optional): I/O binding for the model. Defaults to None.
            **kwargs: Additional keyword arguments passed to the `onnxruntime.InferenceSession`.

        Raises:
            ValueError: If the encoder and decoder models have different types.
            ValueError: If the dtype is not supported.
            ONNXRuntimeError: Exception from ONNXRuntime.
        """
        # inference sessions
        self.encoder_session = ort.InferenceSession(
            encoder_model_path,
            sess_options=session_options,
            providers=provider,
            provider_options=provider_options,
            **kwargs,
        )
        self.decoder_session = ort.InferenceSession(
            decoder_model_path,
            sess_options=session_options,
            providers=provider,
            provider_options=provider_options,
            **kwargs,
        )
        # device type
        providers: list[str] = self.encoder_session.get_providers()
        self.device_type = "cuda" if "CUDAExecutionProvider" in providers else "cpu"
        logger.info(f"Device type: {self.device_type}")
        # warn user if running a quantized model on CUDAExecutionProvider
        model_meta: ort.ModelMetadata = self.encoder_session.get_modelmeta()
        if "quant" in model_meta.producer_name and "CUDAExecutionProvider" in providers:
            logger.warning(
                """
                If you are running a quantized model on CUDAExecutionProvider, you may experience unexpected results or extreme bad performance.
                It is recommended to use float32 or float16 models with CUDAExecutionProvider.
                """
            )
        # dtype
        encoder_dtype: str = self.encoder_session.get_outputs()[0].type
        decoder_dtype: str = self.decoder_session.get_outputs()[0].type
        if encoder_dtype != decoder_dtype:
            err_msg: str = f"Encoder and decoder models type mismatch, encoder: {encoder_dtype}, decoder: {decoder_dtype}"
            logger.error(err_msg)
            raise ValueError(err_msg)
        if self.encoder_session.get_inputs()[0].type == "tensor(float16)":
            self.dtype = np.float16
            logger.info("Using mixed precision model")
        elif self.encoder_session.get_inputs()[0].type == "tensor(float)":
            self.dtype = np.float32
        else:
            err_msg = f"Unsupported dtype: {self.encoder_session.get_inputs()[0].type}"
            logger.error(err_msg)
            raise ValueError(err_msg)
        # io binding
        if use_io_binding is not None:
            self.use_io_binding: bool = use_io_binding
            logger.info(
                f"I/O binding {'enabled' if self.use_io_binding else 'disabled'}"
            )
        else:
            self.use_io_binding = True if self.device_type == "cuda" else False
            logger.info(
                f"I/O binding not specified, using default value `{self.use_io_binding}` for `{self.device_type}`"
            )



import logging
import re


class _CustomFormatter(logging.Formatter):
    grey = "\x1b[38;20m"
    yellow = "\x1b[33;20m"
    red = "\x1b[31;20m"
    bold_red = "\x1b[31;1m"
    reset = "\x1b[0m"
    format_str = (
        "%(asctime)s - %(name)s - %(levelname)s - %(message)s (%(filename)s:%(lineno)d)"
    )

    FORMATS: dict[int, str] = {
        logging.DEBUG: grey + format_str + reset,
        logging.INFO: grey + format_str + reset,
        logging.WARNING: yellow + format_str + reset,
        logging.ERROR: red + format_str + reset,
        logging.CRITICAL: bold_red + format_str + reset,
    }

    def format(self, record: logging.LogRecord) -> str:
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)


logger = logging.getLogger("texifast")
ch = logging.StreamHandler()
ch.setLevel(logging.WARNING)

ch.setFormatter(_CustomFormatter())
logger.addHandler(ch)


def set_log_level(level: int) -> None:
    """Set the log level of the logger.

    Args:
        level (int): The log level to set.

    """
    logger.setLevel(level)


def refine_math_block(text: str) -> str:
    """Refine the math block in the text.

    Args:
        text (str): The text to refine.

    Returns:
        str: The refined text.
    """
    pattern = r"(\$\$.*?\$\$)"
    parts: list[str] = re.split(pattern, text, flags=re.DOTALL)
    last_block_is_math_block: bool = False
    for i in range(len(parts)):
        if re.match(pattern, parts[i]):
            parts[i] = (
                f"$$\n{parts[i][2:-2]}\n$$\n\n"
                if last_block_is_math_block
                else f"\n\n$$\n{parts[i][2:-2]}\n$$\n\n"
            )
            last_block_is_math_block = True
        else:
            parts[i] = parts[i].strip()
    result: str = "".join(parts).strip()
    return result



from importlib.util import find_spec

from .helpers import logger

ERROR_MSG = """
Onnxruntime not found, please ensure you install texifast with the 'cpu' or 'all' extra
like so:

```
pip install texifast[cpu]
# or if you want to use GPU(cuda) acceleration
pip install texifast[gpu]
```

Directly installing texifast will not install onnxruntime by default!!!

```
pip install texifast # don't do this!!!
```
"""


try:
    assert find_spec("onnxruntime") is not None
except AssertionError:
    logger.error(ERROR_MSG)
    raise ImportError(
        "Onnxruntime not found, please install texifast with the 'cpu' or 'gpu' extra"
    )



from pathlib import Path

from huggingface_hub import hf_hub_download

from texifast.model import TxfModel
from texifast.pipeline import TxfPipeline

THIS_DIR = Path(__file__).parent
REVISION = open(THIS_DIR.joinpath("model_revision.txt")).read().strip()


def test_txf_pipeline() -> None:
    encoder_model_path = THIS_DIR.joinpath("encoder_model_quantized.onnx")
    if not encoder_model_path.exists():
        hf_hub_download(
            "Spedon/texify-quantized-onnx",
            filename="encoder_model_quantized.onnx",
            local_dir=THIS_DIR,
            revision=REVISION,
        )
    decoder_model_path = THIS_DIR.joinpath("decoder_model_merged_quantized.onnx")
    if not decoder_model_path.exists():
        hf_hub_download(
            "Spedon/texify-quantized-onnx",
            filename="decoder_model_merged_quantized.onnx",
            local_dir=THIS_DIR,
            revision=REVISION,
        )
    tokenizer_json_path = THIS_DIR.joinpath("tokenizer.json")
    if not tokenizer_json_path.exists():
        hf_hub_download(
            "Spedon/texify-quantized-onnx",
            filename="tokenizer.json",
            local_dir=THIS_DIR,
            revision=REVISION,
        )
    model = TxfModel(
        encoder_model_path=encoder_model_path,
        decoder_model_path=decoder_model_path,
    )
    pipeline = TxfPipeline(model=model, tokenizer=tokenizer_json_path)
    assert (
        pipeline(THIS_DIR.joinpath("latex.png"))
        == "The potential $V_i$ of cell $\\mathcal{C}_i$ centred at position $\\mathbf{r}_i$ is related to the surface charge densities $\\sigma_j$ of cells $\\mathcal{C}_j$ $j\\in[1,N]$ through the superposition principle as: $$V_i\\,=\\,\\sum_{j=0}^{N}\\,\\frac{\\sigma_j}{4\\pi\\epsilon_0}\\,\\int_{\\mathcal{C}_j}\\frac{1}{\\|\\mathbf{r}_i-\\mathbf{r}^{\\prime}\\|}\\,\\mathrm{d}^2\\mathbf{r}^{\\prime}\\,=\\,\\sum_{j=0}^{N}\\,Q_{ij}\\,\\sigma_j,$$ where the integral over the surface of cell $\\mathcal{C}_j$ only depends on $\\mathcal{C}_j$ shape and on the relative position of the target point $\\mathbf{r}_i$ with respect to $\\mathcal{C}_j$ location, as $\\sigma_j$ is assumed constant over the whole surface of cell $\\mathcal{C}_j$. "
    )


def test_txf_pipeline_with_iob() -> None:
    revision = open(THIS_DIR.joinpath("model_revision.txt")).read().strip()
    encoder_model_path = THIS_DIR.joinpath("encoder_model_quantized.onnx")
    if not encoder_model_path.exists():
        hf_hub_download(
            "Spedon/texify-quantized-onnx",
            filename="encoder_model_quantized.onnx",
            local_dir=THIS_DIR,
            revision=revision,
        )
    decoder_model_path = THIS_DIR.joinpath("decoder_model_merged_quantized.onnx")
    if not decoder_model_path.exists():
        hf_hub_download(
            "Spedon/texify-quantized-onnx",
            filename="decoder_model_merged_quantized.onnx",
            local_dir=THIS_DIR,
            revision=revision,
        )
    tokenizer_json_path = THIS_DIR.joinpath("tokenizer.json")
    if not tokenizer_json_path.exists():
        hf_hub_download(
            "Spedon/texify-quantized-onnx",
            filename="tokenizer.json",
            local_dir=THIS_DIR,
            revision=revision,
        )
    model = TxfModel(
        encoder_model_path=encoder_model_path,
        decoder_model_path=decoder_model_path,
        use_io_binding=True,
    )
    pipeline = TxfPipeline(model=model, tokenizer=tokenizer_json_path)
    assert (
        pipeline(THIS_DIR.joinpath("latex.png"))
        == "The potential $V_i$ of cell $\\mathcal{C}_i$ centred at position $\\mathbf{r}_i$ is related to the surface charge densities $\\sigma_j$ of cells $\\mathcal{C}_j$ $j\\in[1,N]$ through the superposition principle as: $$V_i\\,=\\,\\sum_{j=0}^{N}\\,\\frac{\\sigma_j}{4\\pi\\epsilon_0}\\,\\int_{\\mathcal{C}_j}\\frac{1}{\\|\\mathbf{r}_i-\\mathbf{r}^{\\prime}\\|}\\,\\mathrm{d}^2\\mathbf{r}^{\\prime}\\,=\\,\\sum_{j=0}^{N}\\,Q_{ij}\\,\\sigma_j,$$ where the integral over the surface of cell $\\mathcal{C}_j$ only depends on $\\mathcal{C}_j$ shape and on the relative position of the target point $\\mathbf{r}_i$ with respect to $\\mathcal{C}_j$ location, as $\\sigma_j$ is assumed constant over the whole surface of cell $\\mathcal{C}_j$. "
    )



from __future__ import annotations

from pathlib import Path
from typing import TYPE_CHECKING

import numpy as np
import onnxruntime as ort
from PIL import Image, ImageOps
from tokenizers import Tokenizer

from .config import TxfConfig
from .helpers import logger, refine_math_block
from .model import TxfModel

if TYPE_CHECKING:
    from typing import IO, Any, TypeAlias

    from _typeshed import StrOrBytesPath
    from numpy.typing import NDArray

    TxfArray: TypeAlias = NDArray[np.float32 | np.float16]


class TxfPipeline:
    def __init__(
        self,
        model: TxfModel,
        tokenizer: Tokenizer | str | Path,
        config: TxfConfig | None = None,
    ) -> None:
        """Initialize the TxfPipeline class.

        Args:
            model (TxfModel): The model to use for the pipeline.
            tokenizer (Tokenizer | str | Path): The tokenizer to use for the pipeline.
            config (TxfConfig, optional): The configuration to use for the pipeline. Defaults to None.
        """
        # referneces
        self.__model: TxfModel = model
        if isinstance(tokenizer, Tokenizer):
            self.__tokenizer: Tokenizer = tokenizer
        elif isinstance(tokenizer, str):
            self.__tokenizer = Tokenizer.from_file(tokenizer)
        else:
            self.__tokenizer = Tokenizer.from_file(str(tokenizer))
        self.__config: TxfConfig
        if config is not None:
            self.__config = config
        else:
            logger.info("No config provided, using default config")
            self.__config = TxfConfig()

        # token ids
        self.__bos_token_id: np.intp = self.__tokenizer.token_to_id(
            self.__config.bos_token
        )
        self.__eos_token_id: np.intp = self.__tokenizer.token_to_id(
            self.__config.eos_token
        )

        # kv_bindings(for I/O binding)
        self.__kv_bindings: list[tuple[str, str]] | None = None

    def __generate(self, pixel_values: TxfArray, max_new_tokens: int) -> list[np.intp]:
        """Generate tokens from pixel values.

        Note:
            TXFArray is a type alias for numpy array with float32 or float16 dtype.

        Args:
            pixel_values (TxfArray): Pixel values of the image.
            max_new_tokens (int): Maximum number of new tokens to generate.

        Returns:
            list[np.intp]: List of token ids.

        """
        encoder_result: TxfArray = self.__model.encoder_session.run(
            None, {"pixel_values": pixel_values}
        )
        hidden_state: TxfArray = encoder_result[0]

        token_ids: list[np.intp] = [self.__bos_token_id]
        dummy_kv: TxfArray = np.empty((1, 16, 0, 64), dtype=self.__model.dtype)

        decoder_inputs: dict[str, NDArray[Any]] = {
            "encoder_hidden_states": hidden_state,
            "input_ids": np.array([[self.__bos_token_id]], dtype=np.int64),
            "use_cache_branch": np.array([False]),
            **{
                f"past_key_values.{i}.{j}.{k}": dummy_kv
                for i in range(self.__config.decoder_layers)
                for j in ["decoder", "encoder"]
                for k in ["key", "value"]
            },
        }

        while len(token_ids) < max_new_tokens:
            decoder_result: list[NDArray[np.float32]] = (
                self.__model.decoder_session.run(None, decoder_inputs)
            )

            new_token_id: np.intp = np.argmax(decoder_result[0][0, -1, :])
            token_ids.append(new_token_id)
            if new_token_id == self.__eos_token_id:
                break
            if len(token_ids) == 2:  # first pass
                decoder_inputs.update(
                    {
                        "input_ids": np.array([[new_token_id]], dtype=np.int64),
                        "use_cache_branch": np.array([True]),
                        **{
                            f"past_key_values.{i}.{j}.{k}": decoder_result[
                                1 + 4 * i + ik if j == "decoder" else 3 + 4 * i + ik
                            ]
                            for i in range(self.__config.decoder_layers)
                            for j in ["decoder", "encoder"]
                            for ik, k in enumerate(["key", "value"])
                        },
                    }
                )
            else:
                decoder_inputs.update(
                    {
                        "input_ids": np.array([[new_token_id]], dtype=np.int64),
                        **{
                            f"past_key_values.{i}.decoder.{j}": decoder_result[
                                1 + 4 * i + ij
                            ]
                            for i in range(8)
                            for ij, j in enumerate(["key", "value"])
                        },
                    }
                )

        return token_ids

    def __generate_with_io_binding(
        self, pixel_values: TxfArray, max_new_tokens: int
    ) -> list[np.intp]:
        """Generate tokens from pixel values using I/O binding.

        Note:
            TXFArray is a type alias for numpy array with float32 or float16 dtype.

        Args:
            pixel_values (TxfArray): Pixel values of the image.
            max_new_tokens (int): Maximum number of new tokens to generate.

        Returns:
            list[np.intp]: List of token ids.

        """
        # create kv bindings if not exists
        if self.__kv_bindings is None:
            self.kv_bindings = [
                (f"past_key_values.{i}.{j}.{k}", f"present.{i}.{j}.{k}")
                for i in range(self.__config.decoder_layers)
                for j in ["decoder", "encoder"]
                for k in ["key", "value"]
            ]

        # encoder binding
        encoder_io_binding: ort.IOBinding = self.__model.encoder_session.io_binding()
        encoder_io_binding.bind_cpu_input(
            "pixel_values", np.ascontiguousarray(pixel_values)
        )
        encoder_io_binding.bind_output(
            "last_hidden_state", device_type=self.__model.device_type
        )

        # encoder run
        encoder_io_binding.synchronize_inputs()
        self.__model.encoder_session.run_with_iobinding(encoder_io_binding)
        encoder_io_binding.synchronize_outputs()

        # decoder inputs preparation(first pass/use_cache_branch=False)
        hidden_state_ort_value: ort.OrtValue = encoder_io_binding.get_outputs()[0]
        dummy_kv: TxfArray = np.empty((1, 16, 0, 64), dtype=self.__model.dtype)
        token_ids: list[np.intp] = [self.__bos_token_id]
        input_ids_ort_value: ort.OrtValue = ort.OrtValue.ortvalue_from_numpy(
            np.ascontiguousarray(np.array([[self.__bos_token_id]], dtype=np.int64)),
            device_type="cpu",
        )
        use_cache_ort_value: ort.OrtValue = ort.OrtValue.ortvalue_from_numpy(
            np.ascontiguousarray(np.array([False])),
            device_type=self.__model.device_type,
        )
        dummy_kv_ort_value: ort.OrtValue = ort.OrtValue.ortvalue_from_numpy(
            np.ascontiguousarray(dummy_kv), device_type=self.__model.device_type
        )

        # decoder binding
        decoder_io_binding: ort.IOBinding = self.__model.decoder_session.io_binding()
        decoder_io_binding.bind_ortvalue_input(
            "encoder_hidden_states", hidden_state_ort_value
        )
        decoder_io_binding.bind_ortvalue_input("input_ids", input_ids_ort_value)
        decoder_io_binding.bind_ortvalue_input("use_cache_branch", use_cache_ort_value)
        decoder_io_binding.bind_output(
            "logits", device_type="cpu"
        )  # logits will always be on CPU, beacuse it is numpy that orchestrating behind
        for past, present in self.kv_bindings:
            decoder_io_binding.bind_ortvalue_input(past, dummy_kv_ort_value)
            decoder_io_binding.bind_output(
                present, device_type=self.__model.device_type
            )

        # variables
        ekv_ort_values: dict[str, ort.OrtValue] = {}
        first_pass: bool = True

        while len(token_ids) < max_new_tokens:
            # decoder run
            decoder_io_binding.synchronize_inputs()
            self.__model.decoder_session.run_with_iobinding(decoder_io_binding)
            decoder_io_binding.synchronize_outputs()
            decoder_result: list[ort.OrtValue] = decoder_io_binding.get_outputs()

            # update token_ids
            new_token_id: np.intp = np.argmax(decoder_result[0].numpy()[0, -1, :])
            input_ids_ort_value.update_inplace(
                np.ascontiguousarray(np.array([[new_token_id]], dtype=np.int64))
            )
            token_ids.append(new_token_id)
            if new_token_id == self.__eos_token_id:
                break

            # update use_cache_branch
            if first_pass:
                use_cache_ort_value = ort.OrtValue.ortvalue_from_numpy(  # cannot update inplace an OrtValue if it is in a different device
                    np.ascontiguousarray(np.array([True])),
                    device_type=self.__model.device_type,
                )

            # decoder binding(inputs)
            decoder_io_binding.clear_binding_inputs()
            decoder_io_binding.bind_ortvalue_input(
                "encoder_hidden_states", hidden_state_ort_value
            )
            decoder_io_binding.bind_ortvalue_input("input_ids", input_ids_ort_value)

            decoder_io_binding.bind_ortvalue_input(
                "use_cache_branch", use_cache_ort_value
            )
            for idx, (past, present) in enumerate(self.kv_bindings):
                i, j, k = past.split(".")[1:]
                if j == "encoder":
                    if first_pass:
                        ekv_ort_values[f"{i}.{j}.{k}"] = (
                            ort.OrtValue.ortvalue_from_numpy(  # create a copy here because clear_binding_outputs() will remove the original
                                np.ascontiguousarray(decoder_result[1 + idx].numpy()),
                                device_type=self.__model.device_type,
                            )
                        )
                    decoder_io_binding.bind_ortvalue_input(
                        past, ekv_ort_values[f"{i}.{j}.{k}"]
                    )
                else:
                    decoder_io_binding.bind_ortvalue_input(
                        past, decoder_result[1 + idx]
                    )

            # decoder binding(outputs)
            decoder_io_binding.clear_binding_outputs()
            decoder_io_binding.bind_output(
                "logits", device_type=self.__model.device_type
            )
            for _, present in self.kv_bindings:
                decoder_io_binding.bind_output(
                    present, device_type=self.__model.device_type
                )

            # update first_pass
            if first_pass:
                first_pass = False

        return token_ids

    def __preprocess(self, image: Image.Image) -> TxfArray:
        """Preprocess the image.

        Note:
            TXFArray is a type alias for numpy array with float32 or float16 dtype.

        Args:
            image (Image.Image): The image to preprocess as a PIL Image.

        Returns:
            TxfArray: The preprocessed image as a numpy array.

        """
        # RGB
        image = image.convert("RGB")
        # resize
        image.thumbnail(self.__config.size)
        # pad
        delta_width: int = self.__config.size[0] - image.width
        delta_height: int = self.__config.size[1] - image.height
        padding: tuple[int, int, int, int] = (
            delta_width // 2,
            delta_height // 2,
            delta_width - (delta_width // 2),
            delta_height - (delta_height // 2),
        )
        image = ImageOps.expand(image, padding, fill=self.__config.pad_fill_value)
        # rescale
        np_image: TxfArray = (
            np.array(image, dtype=self.__model.dtype) * self.__config.rescale_factor
        )
        # normalize
        mean: TxfArray = np.array(self.__config.image_mean, dtype=self.__model.dtype)
        std: TxfArray = np.array(self.__config.image_std, dtype=self.__model.dtype)
        np_image = (np_image - mean) / std
        # HWC -> CHW -> BCHW
        return np_image.transpose(2, 0, 1).reshape(1, 3, *self.__config.size)

    def __call__(
        self,
        image: Image.Image | StrOrBytesPath | IO[bytes],
        max_new_tokens: int = 384,
        refine_output: bool = False,
    ) -> str:
        """Generate result from the image.

        Args:
            image (Image.Image | StrOrBytesPath | IO[bytes]): The image to generate result from.
            max_new_tokens (int, optional): Maximum number of new tokens to generate. Defaults to 384.
            refine_output (bool, optional): Whether to refine the math block in output. Defaults to False.

        Returns:
            str: The generated result.

        """
        if not isinstance(image, Image.Image):
            image = Image.open(image)
        pixel_values: TxfArray = self.__preprocess(image)
        token_ids: list[np.intp]
        if self.__model.use_io_binding:
            token_ids = self.__generate_with_io_binding(pixel_values, max_new_tokens)
        else:
            token_ids = self.__generate(pixel_values, max_new_tokens)
        return (
            refine_math_block(self.__tokenizer.decode(token_ids))
            if refine_output
            else self.__tokenizer.decode(token_ids)
        )



from __future__ import annotations

from typing import TYPE_CHECKING

import numpy as np

if TYPE_CHECKING:
    from numpy.typing import NDArray

IMAGE_STD: list[float] = [0.229, 0.224, 0.225]
IMAGE_MEAN: list[float] = [0.485, 0.456, 0.406]


class TxfConfig:
    def __init__(
        self,
        size: tuple[int, int] = (420, 420),
        pad_fill_value: int = 255,
        rescale_factor: float = 0.00392156862745098,
        image_std: list[float] = IMAGE_STD,
        image_mean: list[float] = IMAGE_MEAN,
        bos_token: str = "<s>",
        eos_token: str = "</s>",
        decoder_layers: int = 8,
    ) -> None:
        """Initialize the TxfConfig class.

        Note:
            The default values are set based on the `preprocess_config.json` and `config.json` in
            huggingface `Spedon/texify-quantized-onnx` repository.

        Args:
            size (tuple[int, int], optional): Size of the input image. Defaults to (420, 420).
            pad_fill_value (int, optional): Fill value for padding. Defaults to 255.
            rescale_factor (float, optional): Rescale factor for the input image. Defaults to 0.00392156862745098.
            image_std (list[float], optional): Standard deviation of the input image. Defaults to IMAGE_STD.
            image_mean (list[float], optional): Mean of the input image. Defaults to IMAGE_MEAN.
            bos_token (str, optional): Beginning of sentence token. Defaults to "<s>".
            eos_token (str, optional): End of sentence token. Defaults to "</s>".
            decoder_layers (int, optional): Number of decoder layers. Defaults to 8.
        """
        self.size: tuple[int, int] = size
        self.pad_fill_value: int = pad_fill_value
        self.rescale_factor: float = rescale_factor
        self.image_std: NDArray[np.float32] = np.array(image_std, dtype=np.float32)
        self.image_mean: NDArray[np.float32] = np.array(image_mean, dtype=np.float32)
        self.bos_token: str = bos_token
        self.eos_token: str = eos_token
        self.decoder_layers: int = decoder_layers
